// For the Screen
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

// For the Encoder:
#include <Arduino.h>
#include <RotaryEncoder.h>

// To Type
#include <Keyboard.h>

// For pwords
#include "holder-file.h"

// For the Screen
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels


// Pin Declarations for the Screen
//  Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
//  The pins for I2C are defined by the Wire-library.
//  On an arduino UNO:       A4(SDA), A5(SCL)
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// Pin Declarations for the Encoder
//  Example for Arduino UNO with input signals on pin 2 and 3
#define PIN_IN1 0
#define PIN_IN2 1
#define PIN_BUT 7

// Rotary Encoder Setup
// Setup a RotaryEncoder with 2 steps per latch for the 2 signal input pins:
RotaryEncoder encoder(PIN_IN1, PIN_IN2, RotaryEncoder::LatchMode::TWO03);
int button_held = 0;


// This interrupt routine will be called on any change of one of the input signals
void checkPosition()
{
  encoder.tick(); // just call tick() to check the state.
}

void setup() {
  // put your setup code here, to run once:

  // Screen Setup
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    for (;;); // Don't proceed, loop forever
  }
  // Print Lost Screen
  lost();
  holder_setup(); // Run to update the two lists with passwords
  menu(0);

  // Set Up Encoder Interrupts:
  attachInterrupt(digitalPinToInterrupt(PIN_IN1), checkPosition, CHANGE);
  attachInterrupt(digitalPinToInterrupt(PIN_IN2), checkPosition, CHANGE);

  // Button Pin:
  pinMode(PIN_BUT, INPUT_PULLUP);
}

void loop() {
  // put your main code here, to run repeatedly:
  static int pos = 0;
  // Getting values from encoder
  int newPos = encoder.getPosition();
  int encDir = (int) encoder.getDirection();
  if (pos != newPos) {
    // If Position has changed!

    // Preventing Underflow
    if (newPos < 0) {
      newPos += 10 * list_max;
    }

    // Changing Menu Position
    menu(newPos % list_max);

    // Update `pos`
    pos = newPos;
  }

  // Button's been pressed:
  if (!digitalRead(PIN_BUT)) {
    // Do something?

    // first press
    if (button_held == 0) {
      fsu();
      // Print Chars
      Keyboard.print(p_list[newPos % list_max]);
      // Vuage and Basic Debouncing
      delay(200);
      button_held = 1;
    }

    // What to do on every subsequent read?
  }

  // What to do when button is released.
  if (digitalRead(PIN_BUT) && button_held) {
    // Return To Position
    menu(newPos % list_max);
    // Reset `button_held`
    button_held = 0;
  }


}

// Fudge Stuff Up.
//   This is just what to display when typing the password
void fsu() {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(4);             // Full Screen Baby!
  display.println(F("SHHH!"));
  display.display();
  delay(20);
}

void menu(int pnum) {
  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);        // Draw white text
  display.setCursor(0, 0);            // Start at top-left corner

  int pnum_sub; //pnum -1
  int pnum_add; //pnum +1

  // Setting Mins and Maximums
  //int pnum_min = 0;
  int pnum_max = list_max - 1;

  // Fixing Overflow
  if (pnum == 0) {
    pnum_sub = pnum_max;
  } else {
    pnum_sub = pnum - 1;
  }
  if (pnum == pnum_max) {
    pnum_add = 0;
  } else {
    pnum_add = pnum + 1;
  }

  // Top Line
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.print(F("> "));
  display.println(ac_list[pnum_sub]);
  //display.print(F(" = "));          // Debug Lines
  //display.println(pnum_sub, DEC);   // Debug Lines

  // Middle (Large) Line
  display.setTextSize(2);
  display.print(F(">"));
  display.println(ac_list[pnum]);     // Big Middle Line
  //display.print(F(" = "));          // Debug Lines
  //display.println(pnum, DEC);       // Debug Lines

  // Top Line
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.print(F("> "));
  display.print(ac_list[pnum_add]);
  //display.print(F(" = "));          // Debug Lines
  //display.println(pnum_add, DEC);   // Debug Lines

  // Finalise Display
  display.display();
  delay(10);

}

void lost() {
  // Lost Screen:
  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);        // Draw white text
  display.setCursor(0, 0);
  display.println(F("If Lost Please"));
  display.print(F("Text "));
  display.println(F(contact_number)); // Defined in the Holder File
  display.println(F("Kind Regards,"));
  display.println(F(person_name)); // Defined in the holder file
  display.display();
  delay(2000);
}
